/*
 * Copyright (c) 2016, Yuichiro MORIGUCHI
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * * Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * * Neither the name of the Yuichiro MORIGUCHI nor the names of its contributors 
 *   may be used to endorse or promote products derived from this software 
 *   without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Yuichiro MORIGUCHI BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
using System;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Runtime.Serialization;

namespace Morilib.Awk
{
	public class AwkOptions
	{
		private readonly Regex Var = new Regex("^([^=]+)=(.*)$");
		public string FS { get; private set; }
		public string ScriptFileName { get; private set; }
		public string Script { get; private set; }
		private Dictionary<string, string> prevariables = new Dictionary<string, string>();
		private Dictionary<string, string> postvariables = new Dictionary<string, string>();
		private List<string> files = new List<string>();
		private List<string> argv = new List<string>();
		private void CheckOpt(int i, string[] args, string opt)
		{
			if(i + 1 >= args.Length)
			{
				throw new AwkRuntimeException("Illegal option " + opt);
			}
		}
		private bool PutVar(string arg, Dictionary<string, string> dic)
		{
			Match mt;
			mt = Var.Match(arg);
			if(mt.Success)
			{
				dic[mt.Groups[1].Value] = mt.Groups[2].Value;
			}
			return mt.Success;
		}
		public bool ParseOption(string[] args)
		{
			int i = 0;
			for(; i < args.Length; i++)
			{
				if(args[i].Equals("-F"))
				{
					CheckOpt(i, args, "-F");
					FS = args[i + 1];
					i++;
				}
				else if(args[i].Equals("-v"))
				{
					CheckOpt(i, args, "-v");
					PutVar(args[i + 1], prevariables);
					i++;
				}
				else if(args[i].StartsWith("-F"))
				{
					FS = args[i].Substring(2, args[i].Length - 2);
				}
				else if(args[i].Equals("-f"))
				{
					CheckOpt(i, args, "-f");
					ScriptFileName = args[i + 1];
					i++;
				}
				else if(args[i].Equals("--"))
				{
					break;
				}
				else if(args[i].StartsWith("-"))
				{
					// ignore
				}
				else
				{
					break;
				}
			}
			if(ScriptFileName == null)
			{
				if(i >= args.Length)
				{
					return false;
				}
				Script = args[i];
				i++;
			}
			argv.Add("awk");
			for(; i < args.Length; i++)
			{
				argv.Add(args[i]);
				if(!PutVar(args[i], postvariables))
				{
					files.Add(args[i]);
				}
			}
			return true;
		}
		public Dictionary<string, string> GetPreVariables()
		{
			return new Dictionary<string, string>(prevariables);
		}
		public Dictionary<string, string> GetPostVariables()
		{
			return new Dictionary<string, string>(postvariables);
		}
		public List<string> GetFiles()
		{
			return new List<string>(files);
		}
		public List<string> GetArgv()
		{
			return new List<string>(argv);
		}
	}
}
